from .client_base import BaseClient

class Classification(BaseClient):
    def get_classification(self, text: str) -> dict:
        endpoint = "classification-service/classify"
        return self.post(endpoint, {"text": text})


import httpx
from typing import Any, Dict
from urllib.parse import urlparse
import logging

logger = logging.getLogger(__name__)

class BaseClient:
    def __init__(self, mode: str, base_url: str, api_key: str = None, timeout: int = 60):
        # Validate base URL
        # parsed_url = urlparse(base_url)
        # if not parsed_url.scheme or not parsed_url.netloc:
        #     raise ValueError("Invalid base URL provided.")
        
        self.base_url = base_url
        self.api_key = api_key
        self.timeout = timeout
        self.client = httpx.Client(timeout=self.timeout)
        self.mode = mode

    def get(self, endpoint: str, params: Dict[str, Any] = None) -> Any:
        headers = {}
        if self.api_key:
            headers['Authorization'] = f"Bearer {self.api_key}"  # Include API key in headers
        try:
            full_url = f"{self.base_url}{endpoint}"
            response = self.client.get(full_url, params=params, headers=headers)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            print(f"HTTP error occurred: {e} for URL: {full_url}")
            raise
        except httpx.RequestError as e:
            print(f"Request error occurred: {e} for URL: {full_url}")
            raise

    def post(self, endpoint: str, json: Dict[str, Any] = None) -> Any:
        headers = {}
        if self.api_key:
            headers['Authorization'] = f"Bearer {self.api_key}"  # Include API key in headers
        try:
            request = self.client.build_request("POST", f"{self.base_url}{endpoint}", json=json, headers=headers)
            response = self.client.send(request)
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError as e:
            print(f"HTTP error occurred: {e}")
            raise
        except httpx.RequestError as e:
            print(f"Request error occurred: {e}")
            raise

    def close(self):
        self.client.close()


import json
from .client_base import BaseClient
from typing import Dict, List, Optional
from pydantic import BaseModel
from enum import Enum

class Articles(BaseClient):
    """
    Client to interact with the Articles API endpoints.
    """
    def __init__(self, mode: str, base_url: str, api_key: str = None):
        super().__init__(mode, base_url, api_key)

    def __call__(self, *args, **kwargs):
        if args:
            kwargs['query'] = args[0]
        return self.get_articles(*args, **kwargs)
    

    class GetArticlesRequest(BaseModel):
        query: str
        limit: Optional[int] = 10
        from_date: Optional[str] = None
        to_date: Optional[str] = None
        search_type: Optional[str] = None
        entities: Optional[List[str]] = None
        pretty: Optional[bool] = False

    class GetArticlesResponse(BaseModel):
        contents: List[Dict]
        total: int

    def get_articles(self, *args, **kwargs) -> GetArticlesResponse:
        if args and 'query' not in kwargs:
            # If a positional argument is provided and 'query' is not in kwargs, use the first positional argument as 'query'
            kwargs['query'] = args[0]
        
        endpoint = f":5434" + "/v2/contents" if self.mode == "local" else "/postgres-service/contents"
        print(endpoint)
        request = self.GetArticlesRequest(**kwargs)
        params = request.model_dump(exclude={"pretty"})
        print(params)
        if request.pretty:
            # print pretty json
            print(json.dumps(self.get(endpoint, params), indent=4))
        else:
            return self.get(endpoint, params)

    def get_articles_by_entity(self, entity_name: str, skip: int = 0, limit: int = 10) -> Dict:
        endpoint = f"/postgres-service/routes/search/contents_by_entity/{entity_name}"
        params = {"skip": skip, "limit": limit}
        return self.get(endpoint, params)




from .client_base import BaseClient
from typing import List, Optional
from pydantic import BaseModel
import json

class GeoJSON(BaseClient):
    def __init__(self, mode: str, base_url: str, api_key: str = None):
        super().__init__(mode, base_url, api_key)

    def __call__(self, *args, **kwargs):
        if args:
            kwargs['ids'] = args[0]
        return self.by_id(*args, **kwargs)
    
    def get_geojson(self, type: str = "Politics") -> dict:
        endpoint = f"geo-service/geojson_events/{type}"
        return self.get(endpoint)
    
    class ByIdRequest(BaseModel):
        content_ids: List[str]
    
    def by_id(self, ids: List[str], pretty: bool = False) -> dict:
        endpoint = f":3690" + "/geojson_by_content_ids" if self.mode == "local" else "/geo-service/geojson_by_content_ids"
        request = self.ByIdRequest(content_ids=ids)
        params = request.model_dump()
        print("Request payload:", params)
        if pretty:
            print(json.dumps(self.post(endpoint, json=params), indent=4))
        else:
            return self.post(endpoint, json=params)
    

    def by_event(self, event_type: str, limit: Optional[int] = 100, pretty: Optional[bool] = False) -> dict:
        endpoint = f":3690" + f"/geojson_events/{event_type}?limit={limit}" if self.mode == "local" else f"/geo-service/geojson_events/{event_type}"
        if pretty:
            print(json.dumps(self.get(endpoint), indent=4))
        else:
            return self.get(endpoint)   


from .client_base import BaseClient

class Entities(BaseClient):
    """
    Client to interact with the Entities API endpoints.
    """
    def __init__(self, mode: str, base_url: str, api_key: str = None):
        super().__init__(mode, base_url, api_key)
    
    def __call__(self, *args, **kwargs):
        return self.get_entities(*args, **kwargs)

    def get_entities(self, location_name: str, skip: int = 0, limit: int = 50) -> dict:
        endpoint = f"postgres-service/routes/search/location_entities/{location_name}"
        params = {"skip": skip, "limit": limit}
        return self.get(endpoint, params)
    
    def by_id(self, entity_id: str) -> dict:
        endpoint = f"postgres-service/routes/search/entity/{entity_id}"
        return self.get(endpoint)

    def by_location(self, location_name: str, skip: int = 0, limit: int = 50) -> dict:
        endpoint = f"postgres-service/routes/search/location_entities/{location_name}"
        params = {"skip": skip, "limit": limit}
        return self.get(endpoint, params)